//sllist.h

#include <stdlib.h>

#include "sllnode.h"
template <typename T>
class singlyLinkedList
{	public:

		// Constructor
		singlyLinkedList( );
		singlyLinkedList(int n, const T& item=T());
		singlyLinkedList(T* first, T* last);

		// Copy constructor
		singlyLinkedList(const singlyLinkedList & sLLobj);

		// Destructor
		~ singlyLinkedList( );

		// Overloaded assignment operator
		singlyLinkedList<T> & operator = (const singlyLinkedList<T>& sLLobj);

		// List markers
		node<T> * end();
		node<T> * begin();

		// List modifiers
		void push_back(const T& value);
		void pop_back();
    	void push_front(const T& value);
		void pop_front();
		node<T> * insert(node<T> * pos, const T& item);
		void erase(node<T> *pos);
		void erase(node<T> *first, node<T> *last);
		
		// Methods to examine the list
		T & back();
		const T& back()const;
		T & front();
    	const T& front()const;
		bool empty()const;
		int size()const;
		
	private:
		node<T> * head, *rear;
		int listSize;
		
		// private methods
		node<T> * getNode(const T& item);
		node<T> * insertNode(node<T> *curr, const T& item);
		void eraseNode(node<T> * curr);		
};

// getNode allocates a node, stores the data and returns the address
// of the node. The nodes next field is NULL.
template <typename T>
node<T> * singlyLinkedList<T>::getNode(const T& item)
{	
	return (new node<T>(item));
}

// insert will insert a new node whose data field is item, before curr 
// and return the address of the new node.
template <typename T>
node<T> * singlyLinkedList<T>::insertNode(node<T> * curr, const T& item)
{	
	node<T> *prev, *newNode;

	newNode = getNode(item);	// attach tail of list
	newNode->next = curr;

	if (head == curr)  			// insert at front of list
		head = newNode;
	else 						// insert in middle or end of list
	{  	
		prev = head;
		while (prev->next != curr)	
			prev = prev->next;
		prev->next = newNode;
	}

   	if (! curr)	
		rear = newNode;			// update rear if necessary
   	listSize ++;				// update list size
    return newNode;
}

// erase will delete the node whose address is curr from the list
template <typename T>
void singlyLinkedList<T>::eraseNode(node<T> * curr)
{	
	node <T> * prev;
	
	if (curr == head)			// erase front node of list
	{   
		head = head->next;
		if (curr == rear) 		// if list is made empty update rear
			rear = NULL;  
	}	
	else						// erase node in middle or end of list
	{   
		prev = head;			
		while(prev->next != curr)	// locate node in the list
		     prev= prev->next;
		prev->next = curr->next;
		if (curr == rear)		// if erasing at end of list update rear
		     rear = prev;	
	}
	delete curr;			// delete node and update list size
	listSize --;	
}


// default constructor constructs empty list
template <typename T> 
singlyLinkedList<T>::singlyLinkedList( )
{	
	head = rear = NULL;
	listSize = 0;
}

// construct list of n elements all having data value item
template <class T> 
singlyLinkedList<T>::singlyLinkedList(int n, const T& item=T())
{	
	for (int i=1; i<=n; i++)
		head = insertNode(head, item);
}

// construct a sublist of the list from data in the specified range 
template <typename T> 
singlyLinkedList<T>::singlyLinkedList(T* first,  T* last )
{	
	T* curr = first;

	head = new node (*curr);	// create first node in list
	listSize = 1;
	rear = head;
	curr++;

	while (*curr != last)		// insert each subsequent node at end
	{    
		insertNode(NULL, *curr);
	    curr++;
	}
}

// Copy constructor creates a copy of list sLLobj
template <typename T>
singlyLinkedList<T>::singlyLinkedList(const singlyLinkedList<T> & sLLobj)
{	

	node <T> *curr, *newNode;
	
	head = NULL;								// create an empty list
	rear = NULL;
	listSize = 0;

	if (! (sLLobj.empty()))						// copy nodes inserting at rear
	{	
		curr = sLLobj.front();
		head = getNode(*curr);					// copy first node
		curr->next;
		while (curr != sLLobj.end())	        // copy and insert other nodes
		{     
			newNew = insertNode(,NULL, *curr);
		    curr->next;
		}
	}
}

// Overloaded assignment operator creates a copy of list sLLobj
template <typename T>
singlyLinkedList<T> & singlyLinkedList<T>::operator = (const singlyLinkedList<T> & sLLobj)
{	
	node<T> *curr, *newNode;
	
	head = NULL;								// create an empty list
	rear = NULL;
	listSize = 0;
	if (! (sLLobj.empty()))						// copy nodes inserting at rear
	{	
		curr = sLLobj.front();
		head = getNode(*curr);					// copy first node
		curr->next;
		while (curr != sLLobj.end())			// copy and insert other nodes
		{     
			newNew = insertNode(,NULL, *curr);
			curr->next;
		}
	}
	return *this;
}

// destructor deletes all elements in the list
template <typename T>
singlyLinkedList<T>:: ~singlyLinkedList()
{	
	while (!empty())
		eraseNode(head);		// erase front node until list is empty
}

// put the specified value onto the end of the list
template <typename T>
void singlyLinkedList<T>::push_back(const T& value)
{	
	insertNode(NULL, value);
}

// remove the last element in the list
template <typename T>
void singlyLinkedList<T>::pop_back()
{	
	eraseNode(rear);
}

// put the specified value onto the front of the list
template <typename T>
void singlyLinkedList<T>::push_front(const T& value)
{	
	insertNode(head, value);
}

// remove the first element in the list
template <typename T>
void singlyLinkedList<T>::pop_front()
{	
	eraseNode(head);
}

// insert the specified value before the specified position
template <typename T> 
node<T> * singlyLinkedList<T>::insert(node<T> * pos, const T& item)
{	
	node <T> *curr, *newNode;

	curr = pos;
	newNode = insertNode(curr, item);
	return newNode;
}

// remove the element pointed to by pos
template <typename T> 
void singlyLinkedList<T>::erase(node <T>* pos)
{	
	eraseNode(pos);
}

// erase the elements in the range (first, last]
template <typename T> 
void singlyLinkedList<T>::erase(node <T> *first, node <T> *last)
{	
	node<T> *curr, *finish;

	curr = first.nodePtr;
	finish = last.nodePtr;
	while(curr != finish)
		eraseNode(curr);
}

// Retrieve the last item of the list
template <typename T>
T & singlyLinkedList<T>::back()
{	
	return rear->nodeValue;
}

// Retrieve the last item of the list (constant version)
template <typename T>
const T& singlyLinkedList<T>::back()const
{	
	return rear->nodeValue;
}

// Retrieve the first item of the list
template <typename T>
T & singlyLinkedList<T>::front()
{	
	return head->nodeValue;
}

// Retrieve the first item of the list (constant version)
template <typename T>
const T& singlyLinkedList<T>::front()const
{	
	return head->nodeValue;
}

// Retrieve the size of the list
template <typename T>
int singlyLinkedList<T>::size(void) const
{   
	return listSize;
}

// Determines if the list is empty
template <typename T>
bool singlyLinkedList<T>::empty(void) const
{	
	return (listSize == 0);
}

// Return an iterator to the position after the last node in the list
template <typename T>
node<T> * singlyLinkedList<T>:: end()
{	
	return rear->next;		
}

// Return iterator to the first element in the list
template <typename T>
node<T> * singlyLinkedList<T>:: begin()
{	
	return (head);
}

